import sys

from cleo import Command

from framgiaci.common import print_header, read_results


class RunFinishCommand(Command):
    """
    Running finish command tools

    finish
    """

    def handle(self):
        is_good_build = True
        result_text = ''
        for tool, result in read_results(self.app.temp_file_name).items():
            if result['exit_code'] != 0:
                if result['ignore'] == True:
                    format_str = "[!] %s: failed but ignored\n"
                else:
                    format_str = "[x] %s: failed\n"
                    is_good_build = False
            else:
                format_str = "[o] %s: success\n"
            result_text += format_str % tool

        if is_good_build:
            print_header('[o] Build Success!')
            print(result_text)
            sys.exit(0)
        else:
            print_header('[x] Build Fail!')
            print(result_text)
            sys.exit(1)



import sys
import os

from cleo import Command

from framgiaci.common import print_header, run_command, write_results


class RunTestCommand(Command):
    """
    Running test tools

    test
    """

    def handle(self):
        self.app.check_configure_file_exists()
        print_header('Running Test')
        if self.app.ci_reports['test']:
            os.makedirs('.framgia-ci-reports', exist_ok=True)
            test_commands = self.app.ci_reports['test']
            results = {}
            for tool, options in test_commands.items():
                if options.get('enable', True):
                    to_run_cmds = []
                    if isinstance(options['command'], str):
                        to_run_cmds.append(options['command'])
                    elif isinstance(options['command'], list):
                        to_run_cmds = options['command']

                    general_result = 0
                    for command in to_run_cmds:
                        general_result = run_command(command)
                        if general_result:
                            break

                    results[tool] = {
                        'exit_code': general_result,
                        'comment': options.get('comment', True),
                        'ignore': options.get('ignore', False) == True
                    }
            write_results(results, self.app.temp_file_name)
            sys.exit(0)



import os
import yaml
import sys
import pycurl
import json

from io import BytesIO

def run_command(command):
    try:
        print("[+] Running: ", command)
        return os.system(command)

    except Exception as e:
        print('[!] Error:', e)
        return 1


def read_yaml_file(file):
    try:
        with open(file, "r") as f:
            return yaml.load(f.read())
    except Exception as e:
        print('Can not read file', file)
        sys.exit(1)


def read_results(temp_file):
    return read_yaml_file(temp_file)


def write_results(results, temp_file):
    with open(temp_file, 'w') as outfile:
        yaml.dump(results, outfile, default_flow_style=False)


def print_header(text):
    print("\n------------------------------------------")
    print(text)
    print("------------------------------------------\n")

def resource_path(relative_path):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    if getattr(sys, 'frozen', False):
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
    else:
        base_path = os.path.dirname(os.path.realpath(__file__))

    return os.path.join(base_path, relative_path)

def buid_template_file_path(template_dir, file_name):
    return os.path.join(resource_path(template_dir), "%s.yml" % file_name)

def read_template_file(template_dir, file_name):
    return read_yaml_file(buid_template_file_path(template_dir, file_name))

def merge_test_config(base, overwrite):
    if not base:
        return overwrite
    result = {}
    merged_tools = [key for key in overwrite.keys()] + [key for key in base.keys()]
    merged_tools = list(set(merged_tools))
    defaults = {
        'comment': True,
        'ignore': False,
        'enable': True
    }
    for tool in merged_tools:
        if tool not in overwrite:
            result[tool] = base[tool]
        else:
            result[tool] = {}
            for key, value in defaults.items():
                result[tool][key] = overwrite[tool].get(key, base[tool].get(key, defaults[key]))
            result[tool]['command'] = overwrite[tool].get('command', base[tool]['command'])
            for key in [k for k in overwrite[tool].keys() if k not in list(defaults.keys()) + ['command']]:
                result[tool][key] = overwrite[tool][key]

    return result

def build_params():
    repo = os.environ.get('DRONE_REPO').split('/')
    return {
        'workspace': {
            'path': os.environ.get('DRONE_DIR')
        },
        'repo': {
            'owner': repo[0],
            'name': repo[1],
            'full_name': os.environ.get('DRONE_REPO')
        },
        'build': {
            'number': os.environ.get('DRONE_BUILD_NUMBER'),
            'commit': os.environ.get('DRONE_COMMIT'),
            'branch': os.environ.get('DRONE_BRANCH'),
            'pull_request_number': os.environ.get('DRONE_PULL_REQUEST')
        },
        'job': {
            'number': os.environ.get('DRONE_JOB_NUMBER')
        }
    }

def call_api(url, is_post=False, params={}, headers=[], files=[]):
    buffer = BytesIO()
    c = pycurl.Curl()
    c.setopt(c.URL, url)
    c.setopt(c.FOLLOWLOCATION, True)
    c.setopt(c.WRITEDATA, buffer)
    if is_post:
        postfields = []
        for k,v in params.items():
            postfields.append((k, json.dumps(v)))
        if files != []:
            for tag, file in files:
                postfields.append((tag, (c.FORM_FILE, file)))
        c.setopt(c.HTTPPOST, postfields)

    if headers != []:
        c.setopt(c.HTTPHEADER, headers)

    c.perform()
    c.close()

    body = buffer.getvalue()
    body = body.decode('iso-8859-1')
    try:
        return json.loads(body)
    except Exception:
        return {'errorCode': 'Server Error !'}



import os
import sys
from cleo import Application

from framgiaci.common import read_yaml_file, read_template_file, merge_test_config

class ReportApplication(Application):

    PROJ_TYPES = ['php', 'ruby', 'android', 'ios']
    TEMPLATES_DIR = 'templates'

    def __init__(self, *args, **kwargs):
        super(ReportApplication, self).__init__()

    def register_command(self, command_class):
        command = command_class()
        command.app = self
        self.add(command)

    def check_configure_file_exists(self):
        if not os.path.isfile(self.configure_file_name):
            print('.framgia-ci.yml file does not exists')
            sys.exit(1)

    def config(self, configure_file_name, temp_file_name):
        self.configure_file_name = configure_file_name
        self.temp_file_name = temp_file_name
        if os.path.isfile(configure_file_name):
            self.load_ci_reports()

    def load_ci_reports(self):
        if len(sys.argv) == 2 and sys.argv[1] == 'check-config':
            pass
        else:
            self.ci_reports = self.parse_ci_config()
            if 'url' not in self.ci_reports:
                self.ci_reports['url'] = 'https://ci-reports.framgia.vn'

    def parse_ci_config(self):
        raw = read_yaml_file(self.configure_file_name)
        if 'from' in raw:
            base = read_template_file(self.TEMPLATES_DIR, raw['from'])
            final = {'from': raw['from']}
            final['test'] = merge_test_config(base['test'], raw['test'] if 'test' in raw else {})
            final['project_type'] = raw['project_type'] if 'project_type' in raw else base['project_type']

            return final
        else:
            return raw



import os
import sys

from cleo import Command
from framgiaci.common import buid_template_file_path

class InitTemplateCommand(Command):
    """
    Init new config file base-ed on template. Supported project type: php, ruby, android, ios

    init
        {project_type : Supported project type}
        {--short : create compact init file}
    """

    def handle(self):
        project_type = self.argument('project_type')
        if project_type not in self.app.PROJ_TYPES:
            self.line('<error>Invalid project type !</error>')
        else:
            if os.path.isfile(self.app.configure_file_name):
                while True:
                    answer = input('Overwrite file (y/n) ?: ')
                    if answer in ['y', 'n']:
                        break
                    else:
                        self.line("<comment>Please response 'y' or 'n'</comment>")
            else:
                answer = 'y'

            if answer == 'y':
                with open(buid_template_file_path(self.app.TEMPLATES_DIR, project_type), 'r') as fin:
                    with open(self.app.configure_file_name, 'w') as fout:
                        if self.option('short'):
                            fout.write("from: %s" % project_type)
                        else:
                            fout.write(fin.read())
                        self.line("<info>Wrote to file: %s</info>" % self.app.configure_file_name)

        sys.exit(0)



import os
import sys
import time

from cleo import Command
from framgiaci.common import read_results, print_header, call_api, build_params


class RunReportCommand(Command):
    """
    Running report command to send copying request to CI Report service

    report
    """

    def chmod_git_folder():
        os.system('chmod R 755 .git')


    def create_report_queue(self, base_api_url, params):
        print('[+] Creating Report Queue ... ')
        retry_time = 10
        queue_id = None
        token = None
        for i in range(1, retry_time):
            queue_result = call_api(base_api_url, True, params, ['Content-Type: application/json'])
            if (queue_result and 'errorCode' in queue_result and not queue_result['errorCode']):
                queue_id = queue_result['data']['queueId']
                token = queue_result['data']['token']
                break
            else:
                print(i, 'API create report failed!')
                if ('message' in queue_result):
                    print(queue_result['message'])
            time.sleep(5)

        return queue_id, token

    def track_queue(self, queue_id, token, base_api_url):
        retry_time = 10
        print('[+] Tracking queue', queue_id)
        for i in range(1, retry_time):
            time.sleep(5)
            result = call_api(base_api_url + '/' + str(queue_id), False, {}, ['token:' + str(token)])
            if result and 'errorCode' in result and not result['errorCode']:
                if 'error' in result['data'] or 'success' in result['data']['status']:
                    return True if result['data']['status'] == 'success' else False
                else:
                    print(i, 'API check queue status failed!')
                    print(result['data']['status'], result['data']['message'])
                    if ('message' in result):
                        print(result['message'])

        return False

    def handle(self):
        print_header('Sending Reports')
        base_api_url = self.app.ci_reports['url'] + '/api/queues'
        params = build_params()
        params['project_type'] = self.app.ci_reports['project_type'] if 'project_type' in self.app.ci_reports else None
        params['test_result'] = read_results(self.app.temp_file_name)
        queue_id, token = self.create_report_queue(base_api_url, params)
        if queue_id and token:
            self.track_queue(queue_id, token, base_api_url)
        else:
            sys.exit(1)
        sys.exit(0)



import sys
import time
import socket

from cleo import Command


class TestConnectCommand(Command):
    """
    Test connection to specific host and port

    test-connect
        {host : target host}
        {port : target port}
        {timeout? : custom timeout, defautl 120 seconds}
        {--debug : show error each try}
    """

    TIMEOUT = 120

    def try_connect(self, host, port, timeout=1, debug=False):
        try:
            socket.setdefaulttimeout(timeout)
            socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect((host, port))
            return True
        except Exception as e:
            if debug:
                self.line("<comment>%s</comment>" % e)
            return False

    def handle(self):
        host = self.argument('host')
        port = int(self.argument('port'))
        timeout = int(self.argument('timeout')) if self.argument('timeout') else self.TIMEOUT
        debug = self.option('debug')
        self.line("<comment>Try connect to %s:%s (timeout: %s seconds)</comment>" % (host, port, timeout))
        count = 0
        while count < timeout:
            count += 1
            if self.try_connect(host, port, timeout, debug=debug):
                self.line("<info>OK, %d tried</info>" % count)
                break
            time.sleep(1)

        sys.exit(0)



import sys
import os
import yaml

from cleo import Command


class ShowConfigCommand(Command):
    """
    Display current config

    show-config
    """

    def handle(self):
        self.app.check_configure_file_exists()
        print(yaml.dump(self.app.ci_reports, default_flow_style=False))
        sys.exit(0)



import os

from cleo import Command
from framgiaci.common import read_yaml_file, read_template_file, merge_test_config

class CheckConfigCommand(Command):
    """
    Validate config file

    check-config
    """

    ALLOWED_BLOCKS = {
        'command': [str, list],
        'comment': bool,
        'ignore': bool,
        'enable': bool,
    }

    def handle(self):
        self.app.check_configure_file_exists()
        self.line('<info>Start checking...</info>')
        try:
            raw = read_yaml_file(self.app.configure_file_name)
            if 'project_type' not in raw:
                self.line("<comment>No project_type block found !</comment>")

            if 'from' in raw and raw['from'] not in self.app.PROJ_TYPES:
                self.line("<comment>Invalid project type '%s' !</comment>" % raw['from'])
            else:
                base = {}
                extended = {}
                if 'test' not in raw and 'from' not in raw:
                        self.line('<comment>No test block and from block found !</comment>')
                if 'test' in raw:
                    extended = raw['test']
                if 'from' in raw:
                    base = read_template_file(self.app.TEMPLATES_DIR, raw['from'])['test']

                merged_configs = merge_test_config(base, extended)
                for tool, config in merged_configs.items():
                    for key in config.keys():
                        if key not in self.ALLOWED_BLOCKS.keys():
                            self.line("<comment>Block '%s' in tool '%s' is invalid !</comment>" % (key, tool))
                        else:
                            actual = config[key]
                            expected = self.ALLOWED_BLOCKS[key]
                            if isinstance(expected, list):
                                if type(actual) not in expected:
                                    self.line("<comment>Value '%s' of block '%s' in tool '%s' is invalid !</comment>" % (actual, key, tool))
                            elif type(actual) != expected:
                                    self.line("<comment>Value '%s' of block '%s' in tool '%s' is invalid !</comment>" % (actual, key, tool))

        except Exception as e:
            self.line("<comment>Error: %s</comment>" % e)
        finally:
            self.line('<info>Done! Your configuration is valid!</info>')



import sys
import os
import xml.etree.ElementTree as ET
import zipfile
import shutil

from cleo import Command

from framgiaci.common import print_header, build_params, read_results, call_api


class RunUploadCommand(Command):
    """
    Running build and upload reports command

    upload
    """

    ALL_REPORTS = [
        'checkstyle.xml', 'pmd.xml', 'android_lint.xml', # Android
        'rubocop-output-checkstyle.xml', 'reek-output.xml', # Ruby
        'pdepend.xml', 'phpcpd.xml', 'phpcs.xml', 'phpmd.xml', # PHP
        'eslint.xml', # JS
        'swift-lint.xml', # Swift
    ]


    def handle(self):
        print_header('Build and upload reports')
        base_api_url = self.app.ci_reports['url'] + '/api/reports'
        # base_api_url = 'localhost:12345'
        params = build_params()
        params['project_type'] = self.app.ci_reports['project_type'] if 'project_type' in self.app.ci_reports else None
        params['test_result'] = read_results(self.app.temp_file_name)

        self.build_zip_file(params)

        call_api(base_api_url, True, params, [], [('report_file', 'bundle_reports.zip')])
        print('[+] Finish upload')


    def zipdir(self, path, ziph):
        # ziph is zipfile handle
        for root, dirs, files in os.walk(path):
            for file in files:
                ziph.write(os.path.join(root, file), os.path.relpath(os.path.join(root, file), '.framgia-ci-reports'))

    def copytree(self, src, dst, symlinks=False, ignore=None):
        for item in os.listdir(src):
            s = os.path.join(src, item)
            d = os.path.join(dst, item)
            if os.path.isdir(s):
                shutil.copytree(s, d, symlinks, ignore)
            else:
                shutil.copy2(s, d)


    def build_zip_file(self, params, basedir='.framgia-ci-reports'):
        files_list = []
        report_path =  os.path.join(basedir, 'reports')
        os.mkdir(report_path)

        for root, dirs, files in os.walk(basedir):
            for file in files:
                if file in file in self.ALL_REPORTS:
                    xml_report_file = os.path.join(basedir, file)
                    files_list += self.rebuild_and_extract_xml(xml_report_file, params)
                shutil.copy(os.path.join(basedir, file), os.path.join(report_path, file))
            for d in dirs:
                if d != 'reports':
                    os.system("cp -R %s %s" % (os.path.join(basedir, d), report_path))
            break

        files_list = list(set(files_list))

        bundle_zip = zipfile.ZipFile('bundle_reports.zip', 'w')
        for file in files_list:
            full_path = os.path.join(os.getcwd(), file)
            try:
                bundle_zip.write(full_path, os.path.join('src', file), zipfile.ZIP_DEFLATED)
            except Exception as e:
                print('[-]', e)
        self.zipdir(report_path, bundle_zip)

        bundle_zip.close()


    def get_base_root(self, root, file_name):
        if root.tag in ['checkstyle', 'pmd']:

            return root
        if 'pdepend.xml' in file_name:
            parent = [c for c in root if c.tag == 'files']

            return parent[0] if parent else None
        if 'phpcpd.xml' in file_name:
            parent = [c for c in root if c.tag == 'duplication']

            return parent[0] if parent else None
        if 'android_lint.xml' in file_name:

            return [location for issue in root for location in issue]


    def rebuild_and_extract_xml(self, xml_file, params):
        try:
            tree = ET.parse(xml_file)
        except Exception:
            print('[-] Bad XML:', xml_file)
            return []
        root = tree.getroot()
        files = []
        cwd = os.getcwd()
        updated = False
        base_root = self.get_base_root(root, xml_file)
        if not base_root:
            return []
        for child in base_root:
            if child.tag in ['file', 'location']:
                if child.attrib.get('name', None):
                    tag = 'name'
                elif child.attrib.get('path', None):
                    tag = 'path'
                elif child.attrib.get('file', None):
                    tag = 'file'

                fixed_path = child.attrib[tag]
                # iOS work around
                if 'swift-lint.xml' in xml_file:
                    repo = params['repo']['name']
                    fixed_path = fixed_path.split(repo)[-1][1:]
                    updated = True
                elif os.path.isabs(fixed_path):
                    fixed_path = os.path.relpath(fixed_path, cwd)
                    child.set(tag, fixed_path)
                    updated = True
                files.append(fixed_path)

        if updated:
            tree.write(xml_file)

        return files



#!/usr/local/bin/python
# coding: utf-8

import sys
from framgiaci.version import __version__
from framgiaci.report_app import ReportApplication
from framgiaci.commands.run_finish import RunFinishCommand
from framgiaci.commands.run_report import RunReportCommand
from framgiaci.commands.run_test import RunTestCommand
from framgiaci.commands.init_template import InitTemplateCommand
from framgiaci.commands.run_all import RunAllCommand
from framgiaci.commands.check_config import CheckConfigCommand
from framgiaci.commands.show_config import ShowConfigCommand
from framgiaci.commands.run_notify import RunNotifyCommand
from framgiaci.commands.test_connect import TestConnectCommand
from framgiaci.commands.run_upload import RunUploadCommand

YAML_CONFIGURE_FILE = '.framgia-ci.yml'
RESULT_TEMP_FILE = '.framgia-ci-result.temp.yml'

COMMANDS = [
    RunTestCommand, RunReportCommand, RunFinishCommand, InitTemplateCommand,
    CheckConfigCommand, ShowConfigCommand, RunAllCommand, RunNotifyCommand,
    TestConnectCommand, RunUploadCommand
]

def main():
    print('Framgia CI Report Tool', __version__)
    app = ReportApplication()
    app.config(YAML_CONFIGURE_FILE, RESULT_TEMP_FILE)
    for command in COMMANDS:
        app.register_command(command)
    app.run()
    sys.exit(0)

if __name__ == '__main__':
    main()



import sys
import os

from cleo import Command


class RunAllCommand(Command):
    """
    Running test, report, finish command

    run
        {--local : run test and finish only}
    """

    def handle(self):
        self.app.check_configure_file_exists()
        commands = ['test', 'upload', 'finish']
        if self.option('local'):
            commands.remove('upload')
        for command in commands:
            try:
                self.call(command)
            except SystemExit as exception:
                if exception.code:
                    sys.exit(exception.code)

        sys.exit(0)



import sys

from cleo import Command
from framgiaci.common import print_header, build_params, call_api


class RunNotifyCommand(Command):
    """
    Running notify command to send notify request to CI Report service

    notify
    """

    def handle(self):
        print_header('Sending Notifications')
        base_api_url = self.app.ci_reports['url'] + '/api/notifications'
        params = build_params()
        print(call_api(base_api_url, True, params, ['Content-Type: application/json']))
        sys.exit(0)



# -*- coding: utf-8 -*-
"""setup.py: setuptools control."""
from setuptools import setup, find_packages
from distutils.util import convert_path

main_ns = {}
with open(convert_path('framgiaci/version.py')) as ver_file:
    exec(ver_file.read(), main_ns)

setup(
    name = 'framgia-ci',
    packages = ['framgiaci', 'framgiaci.commands'],
    install_requires=['pycurl', 'pyyaml', 'cleo'],
    entry_points = {
        'console_scripts': ['framgia-ci=framgiaci.index:main']
    },
    package_data={'framgiaci': ['templates/*']},
    version = main_ns['__version__'],
    description = 'Command Line Tool for Framgia CI service written in Python',
    author = 'ThangTD and TienNA @Framgia Vietnam CI/CD Team',
    author_email = 'tran.duc.thang@framgia.com',
    url = 'http://ci-reports.framgia.vn',
    classifiers=[
        # Indicate who your project is intended for
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.5',
    ],
    # What does your project relate to?
    keywords='setuptools development framgia-ci',
)